#include <stdio.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <sys/epoll.h>
#include <libmount.h>
#include<sys/epoll.h>


/* *********************************************** 
	使用libmount:
	1. 完成 mount 监控:当有mount信息发生变化的时候，就会打印
	2. 完成mount_info输出:dump_mount_info


 *********************************************** */


static int dump_mount_info(void)
{
	struct libmnt_table *table = NULL;
    struct libmnt_iter *iter = NULL;
	int r = 0;
	
	table = mnt_new_table();
	iter = mnt_new_iter(MNT_ITER_FORWARD);
	if (!table || !iter)
	{
		printf("alloc table/iter failed:%m!\n");
		return -ENOMEM;
	}

	r = mnt_table_parse_mtab(table, NULL);
	
	for (;;) {
		struct libmnt_fs *fs;
		const char *device, *path, *options, *fstype;

		r = mnt_table_next_fs(table, iter, &fs);
		if (r == 1)
			break;
		if (r < 0){
			printf("Failed to get next entry from /proc/self/mountinfo: %m\n");
			goto fail;
		}

		device = mnt_fs_get_source(fs);
		path = mnt_fs_get_target(fs);
		options = mnt_fs_get_options(fs);
		fstype = mnt_fs_get_fstype(fs);

		if (!device || !path)
			continue;
		printf("get mount info:device:%s;path:%s;options:%s;fstype:%s;\n",
									device, path, options, fstype);
	}

	return 0;
	
fail:
	return r;
}

static int monitor_mount_info(void)
{
	struct libmnt_monitor *mount_monitor = NULL;
	int r = 0;
	int monitor_fd = 0;
	char *filename = NULL;
	
	struct epoll_event ev = {0};
	int epoll_fd;
		
	mount_monitor = mnt_new_monitor();
	if (!mount_monitor)
	{
		printf("Failed to new monitor: %m\n");
		return -ENOMEM;
	}
	
	r = mnt_monitor_enable_kernel(mount_monitor, 1);
	if (r < 0)
	{
		printf("Failed to enable watching of kernel mount events: %m\n");
		goto fail;
	}
	
	printf("waiting for changes...\n");
	while (mnt_monitor_wait(mount_monitor, -1) > 0)
	{
		while (mnt_monitor_next_change(mount_monitor, &filename, NULL) == 0)
			printf(" %s: change detected\n", filename);
	}

	mnt_unref_monitor(mount_monitor);

	return 0;
fail:
	mnt_unref_monitor(mount_monitor);
	return r;
}



int main(void)
{
	dump_mount_info();
	monitor_mount_info();

}
